package edu.neu.ccs.beans.demo.colorbeans;

import java.util.*;
import java.io.*;
import java.awt.*;
import java.awt.event.*;
import java.beans.*;
import java.beans.beancontext.*;


/**
 * Class ColorBeanContext behaves a BeanContext that only beans matching
 * the specific criterion(in this case, the criterion is Color) can be
 * added in. If any bean living in the ColorBeanContext has been changed
 * its color, it will notify the ColorBeanContext to check if the bean
 * is still matching the criterion color. If not, the bean will be removed
 * from the ColorBeanContext. If the criterion color of ColorBeanContext
 * is changed, the ColorBeanContext will remove all those beans that
 * no longer match the criterion from itself.
 * 
 */
public class ColorBeanContext extends Panel implements BeanContextProxy {
	private final static Dimension size = new Dimension(200,180);
	private static Color background = Color.red;
	private static Color foreground = Color.black;

	private final transient Object addLock = new String("");
	private final transient Object removeLock = new String("");	
	private transient boolean adding = false;
	private transient boolean removing = false;

	private BeanContextSupport colorBeanContext = null;
	private Color criterionColorOfBean = background;



	/**
	 * Whenever the bean's color is changed, the bean will notify
	 * this PropertyChangeListener(s), and method propertyChange
	 * will be called.
	 * When that happens, method propertyChange will check the bean, which
	 * has sent PropertychangeEvent, if its color is the criterion
	 * color. If not, remove the bean from this panel, and add to its parent.
	 * @param  pce PropertyChange Event
	 */
	private class ColorListener implements PropertyChangeListener, Serializable {
		public void propertyChange(PropertyChangeEvent pce) {
			ColorBean cb = (ColorBean)pce.getSource();
			if (!checkBeanColor(cb)) {
					remove(cb);
					if (getParent() != null)
						getParent().add(cb);
				}
			} // end of propertyChange

	}
	private ColorListener colorListener = new ColorListener();
	/**
	 * Constructor
	 * 
	 */
	public ColorBeanContext() {
		setName("ColorBeanContext");
		setSize(size);
		setForeground(foreground);
		setBackground(background);
		
		colorBeanContext = new BeanContextSupport() {
			protected boolean validatePendingAdd(Object obj) {
				ColorBean colorBean = null;

				// check if the new child is a ColorBean
				try {
					colorBean = (ColorBean) obj;
				} catch (ClassCastException e) {
					return false;
				}
					
				return checkBeanColor(colorBean);
			}

			protected void childJustAddedHook(Object child,
								  BeanContextSupport.BCSChild bcsc) {
				ColorBean colorBean = (ColorBean) child;
				// register this beancontext as the bean's property change 
				// listener
				colorBean.addColorListener(colorListener);
						
				// add bean to the Panel
				synchronized(addLock) {
					if (!adding) {
						adding = true;
						ColorBeanContext.this.add(colorBean);
						adding = false;
					}
				}
			 	repaint();
			} 	
			
			protected void childJustRemovedHook(Object child,
									BeanContextSupport.BCSChild bcsc) {
				ColorBean colorBean = (ColorBean)child;
				// unregister this beancontext as the bean's property change 
				// listener
				colorBean.removeColorListener(colorListener);
					
				synchronized(removeLock) {
					if (!removing) {
						removing = true;
						ColorBeanContext.this.remove(colorBean);
						removing = false;
					}
				}
			 	repaint();
			}
		};
	}
/**
 * Insert the method's description here.
 * Creation date: (2/21/00 4:54:51 PM)
 * @param comp java.awt.Component
 * @param constraints java.lang.Object
 * @param index int
 */
protected void addImpl(Component comp, Object constraints, int index) {
	if (isAncestorOf(comp))
		return;
		
	boolean success = true;
	synchronized(addLock) {
		if (!adding) {
			adding = true;
			success = colorBeanContext.add(comp);
			adding = false;
		}
	}
	if (success)
		super.addImpl(comp, constraints, index);

 	repaint();
}
	/** 
	 * Private method for checking if the bean's color is the
	 * criterion color.
	 *
	 * @param cb ColorBean object
	 *
	 * @return  Boolean value saying if the bean's color matches
	 *           the criterion color
	 */
	private boolean checkBeanColor(ColorBean cb) {
		return cb.getColor() != criterionColorOfBean;
	}
	/** 
	 * Internal method to draw all the beans inside colorBeanContext to this
	 * component.
	 *
	 */
	private void drawBeans(Graphics g) {
		int x = 5, y = 5;
		int maxY = this.getSize().height;
		int fontSize = getFont().getSize();
		
		y += fontSize;
		
		if (colorBeanContext.isEmpty()) {
			g.drawString(this.getName() + " is empty!", x, y);
		}
		else {
			g.drawString(this.getName() + " is now containing:", x, y);
			y += fontSize + 7;
			x += 7;
			
			for (Iterator it = colorBeanContext.iterator(); it.hasNext(); ) {
				ColorBean cb = (ColorBean)it.next();
				
				if (y < (maxY - 30)) {
					g.drawString(cb.getName(), x, y);
					y += fontSize + 3;
				}
				else {
					g.drawString("...", x, (y - 5));
					break;
				}
				cb.repaint();
			}
		}
	}
	/** 
	 * This method is for the BeanContextProxy
	 *
	 */
	public BeanContextChild getBeanContextProxy() {
		return colorBeanContext;
	}
	/**
	 * This method is used to get BeanContext's color.
	 *
	 * @return      a Color object
	 *
	 */
	public Color getColor() {
		return getBackground();
	}
	/**
	 * This method specifies that the preferred display size of the
	 * bean should be 300x270.
	 *
	 * @return      Preferred display size of the bean.
	 */
	public Dimension getPrefferedSize() {
		return size;
	}
	/** 
	 * Overide the paint method. This method is called when the contents 
	 * of the component should be painted in response to the component 
	 * first being shown or damage needing repair.
	 *
	 */
	public void paint(Graphics g) {
		drawBeans(g);
	}
/**
 * Insert the method's description here.
 * Creation date: (2/21/00 4:57:42 PM)
 * @param index int
 */
public void remove(int index) {

	ColorBean colorBean = (ColorBean) getComponent(index);
	boolean success = true;

	synchronized(removeLock) {
		if (!removing) {
			removing = true;
			success = colorBeanContext.remove(colorBean);
			removing = false;
		}
	}

	if (success)
	  super.remove(index);
 	repaint();
}
	/**
	 * This method is used to set BeanContext's color.
	 * Everytime the BeanContext's color is changed, it will remove those
	 * beans that live inside it.
	 * 
	 * @param       newColor    a Color object
	 *
	 */
	public void setColor(Color newColor) {
		Color oldColor = criterionColorOfBean;
		if (newColor == oldColor)
			return;
			
		setBackground(newColor);
		criterionColorOfBean = newColor;
		
		Container parent = getParent();

		Object bean[] = colorBeanContext.toArray();
		for(int i = bean.length - 1; i >= 0; --i) {
			ColorBean cb = (ColorBean)bean[i];
			if (!checkBeanColor(cb)) {
				remove(cb);
				if (parent != null)
					getParent().add(cb);
				
			} // end of propertyChange
		}
 	    repaint();
	}
} // end of class
